/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */

#include "thunder-nautilus-view.h"

#include "thunder-box.h"

#include <libnautilus/nautilus-bonobo-ui.h>
#include <bonobo/bonobo-control.h>
#include <libgnomevfs/gnome-vfs.h>

#include <libgnome/gnome-i18n.h>
#include <libgnomeui/gnome-stock.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk-pixbuf/gdk-pixbuf-loader.h>
#include <gtk/gtklabel.h>
#include <gtk/gtksignal.h>

#include <stdio.h>

struct ThunderNautilusViewDetails {
	GtkWidget *thunder_box;
};

static void thunder_nautilus_view_initialize_class (ThunderNautilusViewClass *klass);
static void thunder_nautilus_view_initialize       (ThunderNautilusView      *view);
static void thunder_nautilus_view_destroy          (GtkObject                      *object);
static void thunder_load_location_callback         (NautilusView                   *thunder_view,
						    const char                     *location,
						    gpointer                        user_data);
static void thunder_merge_bonobo_items_callback    (BonoboControl                  *control,
						    gboolean                        state,
						    gpointer                        user_data);



static gpointer parent_class;
                                                                                                        
GtkType
thunder_nautilus_view_get_type (void)                                                         
{                                                                                                       
	GtkType parent_type;                                                                            
	static GtkType type;                                                                            
                                                                                                        
	if (type == 0) {                                                                                
		static GtkTypeInfo info = {
		        "ThunderNautilusView",
			sizeof (ThunderNautilusView),
			sizeof (ThunderNautilusViewClass),
			(GtkClassInitFunc)thunder_nautilus_view_initialize_class,
			(GtkObjectInitFunc)thunder_nautilus_view_initialize,
			NULL,
			NULL,
			NULL
		};

		parent_type = (NAUTILUS_TYPE_VIEW);
		type = gtk_type_unique (parent_type, &info);
		parent_class = gtk_type_class (parent_type);
	}

	return type;
}


     
static void
thunder_nautilus_view_initialize_class (ThunderNautilusViewClass *klass)
{
	GtkObjectClass *object_class;
	
	g_assert (THUNDER_IS_NAUTILUS_VIEW_CLASS (klass));

	object_class = GTK_OBJECT_CLASS (klass);
	
	object_class->destroy = thunder_nautilus_view_destroy;
}

static void
thunder_nautilus_view_initialize (ThunderNautilusView *view)
{
	g_assert (THUNDER_IS_NAUTILUS_VIEW (view));

	view->details = g_new0 (ThunderNautilusViewDetails, 1);
	
	view->details->thunder_box = thunder_box_new ();
	
	gtk_widget_show (GTK_WIDGET (view->details->thunder_box));

	nautilus_view_construct (NAUTILUS_VIEW (view), 
				 view->details->thunder_box);
	
	gtk_signal_connect (GTK_OBJECT (view), 
			    "load_location",
			    thunder_load_location_callback, 
			    NULL);

	/* Get notified when our bonobo control is activated so we can
	 * merge menu & toolbar items into the shell's UI.
	 */
        gtk_signal_connect (GTK_OBJECT (nautilus_view_get_bonobo_control (NAUTILUS_VIEW (view))),
                            "activate",
                            thunder_merge_bonobo_items_callback,
                            view);
	
}

static void
thunder_nautilus_view_destroy (GtkObject *object)
{
	ThunderNautilusView *view;
	
	view = THUNDER_NAUTILUS_VIEW (object);
	
	g_free (view->details);

	(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static GdkPixbuf *
load_image (ThunderBox *box,
	    const char *filename,
	    gpointer data)
{
	char *dirname = data;
	GnomeVFSHandle *handle;
	char buf[256];
	GnomeVFSResult result;
	GnomeVFSFileSize bytes_read;
	char *fullname;
	GdkPixbuf *pixbuf;
	GdkPixbufLoader *pixbuf_loader;
	GnomeVFSURI *uri, *fulluri;

	g_assert (THUNDER_IS_BOX (box));

	uri = gnome_vfs_uri_new (dirname);
	fulluri = gnome_vfs_uri_append_file_name (uri, filename);
	gnome_vfs_uri_unref (uri);
	fullname = g_strdup (gnome_vfs_uri_get_path (fulluri));
	gnome_vfs_uri_unref (fulluri);

	if (fullname == NULL)
		return NULL;

	result = gnome_vfs_open (&handle,
				 fullname,
				 GNOME_VFS_OPEN_READ);

	g_free (fullname);

	if (result != GNOME_VFS_OK)
		return NULL;

	pixbuf_loader = gdk_pixbuf_loader_new ();
	
	while ((result = gnome_vfs_read (handle, buf,
					 sizeof (buf),
					 &bytes_read)) == GNOME_VFS_OK) {
		gdk_pixbuf_loader_write (pixbuf_loader, buf, bytes_read);
	}

	gnome_vfs_close (handle);

	pixbuf = gdk_pixbuf_loader_get_pixbuf (pixbuf_loader);
	if (pixbuf != NULL)
		gdk_pixbuf_ref (pixbuf);
	gtk_object_unref (GTK_OBJECT (pixbuf_loader));

	return pixbuf;
}


static void
load_location (ThunderNautilusView *view,
	       const char *location)
{
	GnomeVFSHandle *handle;
	char buf[256];
	ThunderPresentation *presentation;
	GnomeVFSResult result;
	GnomeVFSFileSize bytes_read;
	GnomeVFSURI *uri;
	char *dirname;


	g_assert (THUNDER_IS_NAUTILUS_VIEW (view));
	g_assert (location != NULL);

	result = gnome_vfs_open (&handle,
				 location,
				 GNOME_VFS_OPEN_READ);

	if (result != GNOME_VFS_OK) {
		nautilus_view_report_load_failed (NAUTILUS_VIEW (view));
		return;
	}
	
	presentation = thunder_presentation_new ();
	
	while ((result = gnome_vfs_read (handle, buf,
					 sizeof (buf) - 1,
					 &bytes_read)) == GNOME_VFS_OK) {
		buf[bytes_read] = '\0'; 
		thunder_presentation_add_xml (presentation, buf);
	}

	gnome_vfs_close (handle);

	if (result != GNOME_VFS_ERROR_EOF) {
		nautilus_view_report_load_failed (NAUTILUS_VIEW (view));
		gtk_object_unref (GTK_OBJECT (presentation));
		return;
	}

	thunder_presentation_parse_xml (presentation);

	
	uri = gnome_vfs_uri_new (location);
	dirname = gnome_vfs_uri_extract_dirname (uri);
	gnome_vfs_uri_unref (uri);

	gtk_signal_connect_full (GTK_OBJECT (view->details->thunder_box),
				 "load_image",
				 GTK_SIGNAL_FUNC (load_image),
				 NULL,
				 dirname, (GtkDestroyNotify) g_free,
				 FALSE, FALSE);

	thunder_box_set_presentation (THUNDER_BOX (view->details->thunder_box),
				      presentation);

	gtk_object_unref (GTK_OBJECT (presentation));

	nautilus_view_report_load_complete (NAUTILUS_VIEW (view));
}


static void
thunder_load_location_callback (NautilusView *thunder_view, 
				const char *location,
				gpointer user_data)
{
	ThunderNautilusView *view;
	
	g_assert (NAUTILUS_IS_VIEW (thunder_view));
	g_assert (location != NULL);
	
	view = THUNDER_NAUTILUS_VIEW (thunder_view);
	
	nautilus_view_report_load_underway (thunder_view);
	
	load_location (view, location);
}

static void
bonobo_thunder_callback (BonoboUIComponent *ui, 
			 gpointer           user_data, 
			 const char        *verb)
{
 	ThunderNautilusView *view;
	const char *dialog_text;

	g_assert (BONOBO_IS_UI_COMPONENT (ui));
        g_assert (verb != NULL);

	view = THUNDER_NAUTILUS_VIEW (user_data);

	if (strcmp (verb, "Thunder Menu Item") == 0) {
		dialog_text = "You selected the Thunder menu item.";
	} else {
		g_assert (strcmp (verb, "Thunder Dock Item") == 0);
		dialog_text = "You clicked the Thunder toolbar button.";
	}
	
	gnome_ok_dialog (dialog_text);
}

static void
thunder_merge_bonobo_items_callback (BonoboControl *control, 
				    gboolean       state, 
				    gpointer       user_data)
{
 	ThunderNautilusView *view;
	BonoboUIComponent *ui_component;
	BonoboUIVerb verbs [] = {
		BONOBO_UI_VERB ("Thunder Menu Item", bonobo_thunder_callback),
		BONOBO_UI_VERB ("Thunder Dock Item", bonobo_thunder_callback),
		BONOBO_UI_VERB_END
	};

	g_assert (BONOBO_IS_CONTROL (control));
	
	view = THUNDER_NAUTILUS_VIEW (user_data);

	if (state) {
		ui_component = nautilus_view_set_up_ui (NAUTILUS_VIEW (view),
							DATADIR,
							"thunder-nautilus-view-ui.xml",
							"thunder-nautilus-view");
									
		bonobo_ui_component_add_verb_list_with_data (ui_component, verbs, view);
	}

        /* Note that we do nothing if state is FALSE. Thunder content
         * views are activated when installed, but never explicitly
         * deactivated. When the view changes to another, the content
         * view object is destroyed, which ends up calling
         * bonobo_ui_handler_unset_container, which removes its merged
         * menu & toolbar items.
	 */
}

